
/*********************************************************
 * ScreenTool for MRETV  whmp 2013 7 30
 *
 * ScreenTool is a flexible screen environment you can change in real time.
 * It makes determining the screen settings for your own sketches simple. 
 * It is also helps learn screen settings and explore combinations easily.
 * You can browse memory and check audio 
 * 
 * for MasteRasterExtendedTransmissionVision 1
 **********************************************************/

#include <MrETV.h>
#ifndef ScreenTool // make sure the extensions are set correctly
#error Please define ScreenTool in arduino\libraries\MRETV\MRETV.extensions 
#endif
/* *****************  defined in arduino\libraries\MRETV\MRETV.extensions
 #define firmwaremultiport 
 #define mergstreams 
 #define dualaudio   
 #define translatekeys 
 #define fontheight 8  
 #define graphicsengine        // virtual graphics processor
 #define countframes   // for a timer
 ************** */

#include <EEPROM.h>     // to be able to peek at all mem
#include "textfont.h"   //plain font
#include "shootemupfont.h" // a fancier font
#include "saw.h"      // sound

//  #define __AVR_ATmega168__   //testing

PROGMEM const char * sounds [] = 	{
  silent,
  #ifdef __AVR_ATmega168__
#else
  sawCo,sawDbo,sawDo,sawEbo,sawEo,sawFo,sawGbo,sawGo,sawAbo,sawAo,sawBbo,sawBo,
  sawCo1,sawDbo1,sawDo1,sawEbo1,sawEo1,sawFo1,sawGbo1,sawGo1,sawAbo1,sawAo1,sawBbo1,sawBo1,
  sawCo2,sawDbo2,sawDo2,sawEbo2,sawEo2,sawFo2,sawGbo2,sawGo2,sawAbo2,sawAo2,sawBbo2,sawBo2,
  sawCo3,sawDbo3,sawDo3,sawEbo3,sawEo3,sawFo3,
  #endif
  sawGbo3,sawGo3,sawAbo3,sawAo3,sawBbo3,sawBo3,
  sawCo4,sawDbo4,sawDo4,sawEbo4,sawEo4,sawFo4,sawGbo4,sawGo4,sawAbo4,sawAo4,sawBbo4,sawBo4,
  sawCo5,sawDbo5,sawDo5,sawEbo5,sawEo5,sawFo5,sawGbo5,sawGo5,sawAbo5,sawAo5,sawBbo5,sawBo5,
  sawCo6,sawDbo6,sawDo6,sawEbo6,sawEo6,sawFo6,sawGbo6,sawGo6,sawAbo6,sawAo6,sawBbo6,sawBo6,
  sawCo7,sawDbo7,sawDo7,sawEbo7,sawEo7,sawFo7,sawGbo7,sawGo7,sawAbo7,sawAo7,sawBbo7,sawBo7
};



// at the end of this file is a simpler example of 'big line'
// 'fixbigline' has to deal with a very flexible screen definition

volatile uchar fixbigline[]=  // the graphics processor instructions as an array
{
  // instruction format is  line#,flags,value(,value2...),
  // at line 95 set screen w=22 and character w=1 h=2, in operation the set function changes the line count. 
  95,flag_gopcharwidth|flag_gopscreenwidth|flag_gopcharheight,1,22, 2,  
  // at line 111 set  screen w and character w,h also index since we changed screen width
  //(index=index into character buffer of first char next line.)
  111,flag_gopcharwidth|flag_gopindex |flag_gopscreenwidth|flag_gopcharheight,0,5,5,15, 1,   
  // at line 255 reset to first instruction
  255,flag_gopreset ,gop_int(fixbigline)
  };

  //see simpler function at end of this file
  //change the coprocessor instructions to enlarge selected line in a changing screen environment, 
  volatile uchar * setfixbigline( uint textbase, uchar linelen, uchar selline,uchar cw,uchar ch){
    Screen.set((Video.ScreenBottom-Video.ScreenTop+7-8)/8,linelen); //+7 rounding -8 double height line
    while( Video.LineCount< Video.ScreenBottom);  //wait until we are off screen
    fixbigline[0]= Video.ScreenTop+(selline*8*ch)-1; // target line
    //fixbigline[1]=flag_gopcharwidth|flag_gopscreenwidth|flag_gopcharheight;
    fixbigline[2]=cw?0:1; //double wide char
    fixbigline[3]=(linelen+( Video.ScreenCenter/8)<23)?linelen:23-( Video.ScreenCenter/8);
    fixbigline[4]=(ch==1)?2:1; // double tall chars
    fixbigline[5]=Video.ScreenTop+(selline*8*ch)+(ch==1?15:7); // end of target line
    // fixbigline[6]=flag_gopcharwidth|flag_gopindex |flag_gopscreenwidth|flag_gopcharheight;
    fixbigline[7]=cw; //single wide
    fixbigline[8]=(textbase+((selline+1)*linelen))&0xff; // skipped half a line so fix pointer
    fixbigline[9]=(textbase+((selline+1)*linelen))>>8;
    fixbigline[10]=linelen;
    fixbigline[11]=ch; // char height
    //fixbigline[12]=5;//+256 =line261 good time to reset list
    //fixbigline[13]=flag_gopreset;
    //fixbigline[14]=(uchar)((int)fixbigline&0xff); // start of list
    //fixbigline[15]=(uchar)((int)fixbigline>>8);
    return(fixbigline);
  } 
#ifdef __AVR_ATmega168__
uchar  Screenbuf[(25*34)]; // character data
#else
#ifdef __AVR_ATmega2560__
uchar  Screenbuf[(60*50)]; // extra room to grow
#else
uchar  Screenbuf[(30*50)]; // room to grow
#endif
#endif
void setup(){ 
}
void loop() {
  uchar cw=0,ch=1,memmode=0,audio=0,chin=0,keyline=0;
  int memory=0;
  char twidth; // 
   
#ifdef __AVR_ATmega168__
  twidth=34;
  Screen.begin(textfont,Screenbuf, 24*fontheight,twidth, 28,49,1); // 23 + 1 for double high select line.
#else 
  twidth=42;
  Screen.begin(textfont,Screenbuf, 24*fontheight,twidth, 28,9,1); // 23 + 1 for double high select line.
#endif
  Screen.fill(32); // clear screen
  setfixbigline((uint)Screenbuf,twidth,1+keyline*2,cw,ch); // set selection indication line values
  setgoplist(fixbigline);
  while(1)
  { 
    if(chin||(!(Video.FrameCount&15))){ //every 1/4 sec or if there is a character
      char b;
      // try to keep screen printing routines updated to real screen size..
      Screen.set(((Video.ScreenBottom- Video.ScreenTop)/8)+1,twidth); 
      if(chin)    Screen.hex(1,0,chin);  // show character code
      Screen.write(0,0,8);               // and other stuff
      Screen.write(( Video.ScreenBottom- Video.ScreenTop-9)/8,0,8);
      Screen.write(( Video.ScreenBottom- Video.ScreenTop-9)/8,twidth-1,8);
      Screen.write(0,twidth-1,8);
#ifdef __AVR_ATmega168__
      Screen_progstr(0,1,"ScreenTool ^v<>+-pdeir <sft> a-Z");
#else
      Screen_progstr(0,1," GOPScreenTool ^v<>+-pdeir <sft> a-Z");
#endif
Screen_progstr(3,0, "Left");
      Screen.hex(3,7, Video.ScreenCenter);
      Screen_progstr(5,0, "Pix Width");
      Screen.hex(5,12,cw);
      Screen_progstr(7,0, "Pix Height");
      Screen.hex(7,12,ch);
      Screen_progstr(9,0, "Screen Width");
      Screen.hex(9,13,Video.ScreenWidth);
      Screen_progstr(11,0,"Screen Top");
      Screen.hex(11,11,Video.ScreenTop);
      Screen_progstr(13,0,"Screen Bottom");
      Screen.hex(13,14,Video.ScreenBottom);
      Screen.hex(13,18,( Video.ScreenBottom- Video.ScreenTop));
      Screen_progstr(15,0,"Audio");
      Screen.hexint(15,6,( int) Video.audiodata);
      Screen.hexint(15,11,( int) Video.audiodatar);
      Screen.hex(15,16 ,audio);
      Screen.hex(15,19,OCR0A);
      Screen_progstr(17,0,"Gop");
      Screen.hexint(17,4,( int)Video.graphicsoperations);
      Screen.hex(17,9,( int)OCR2A);
      switch(memmode)  {            // show memoery contents
      case 0: //prog
        Screen_progstr(19,0,"Prog Memory");
        Screen.hexint(19,12,( int)memory);
        for(b=0;b<8;b++)      Screen.hex(21,b*3,(int)pgm_read_byte(memory+b));
        break;
      case 1: // data
        Screen_progstr(19,0,"Data Memory");
        Screen.hexint(19,12,( int)memory);
        for(b=0;b<8;b++)        Screen.hex(21,b*3,*((char *)(memory+b)));
        break;
      case 2: // eprom
        Screen_progstr(19,0,"EEprom Mem ");
        Screen.hexint(19,12,( int)memory);
        for(b=0;b<8;b++)        Screen.hex(21,b*3,(char) EEPROM.read(memory+b));
        break;
      } //switch(memmode)
    }
    if(!(chin=Video.readS())){   // no input
      while( Video.LineCount< Video.ScreenBottom); // wait for off Screen
      Screen.time(1,3);               // print time from count frames
      if(key_sft) Video.CharSet=(uchar *)shootemupfont;   // check shift here because
      else Video.CharSet=(uchar *)textfont;  //  it doesn't show in the stream, just colors other characters
    }
    else {   // handle the input
      if(keyline==7){
 #ifdef __AVR_ATmega168__
       if(chin>='a'&&chin<='z')setaudior(chin-'a'+1,60);
        if(chin>='A'&&chin<='Z')setaudiol(chin-'A'+1,60);
  #else
         if(chin>='a'&&chin<='z')setaudior(chin-'a'+31,60);
        if(chin>='A'&&chin<='Z')setaudiol(chin-'A'+31,60);

#endif
}
      if(keyline==3){
        if(chin=='i')Video.FnHSync^=1;
      }
       switch(chin){
       case 'r':
       case 'R':
        asm volatile("JMP 0x0000\n\t"::);
        // break; never going to get here.
      case 0x2B:
        if(keyline==9)  memory+=0x100;
        break;
      case 0x2d:
        if(keyline==9)  memory-=0x100;
        break;
      case 'P':
         if(keyline==9)  memmode=0;
        break;
      case 'd':
        if(keyline==9)   memmode=1;
        break;
     case 'e':
         if(keyline==9)    memmode=2;
        break;
      case 0x42 : //dwn
      case 0x32 : //dwn
        keyline++;
        break;
      case 0x41 :  //up
      case 0x38 :  //up
        keyline+=9;  
        break;
      case 0x43 :  //right
      case 0x44 :  //left
      case 0x36 :  //right
      case 0x34 :  //left
  if(chin==0x36)chin=0x43;    // handle caplock
  else if(chin==0x34)chin=0x44;   
  switch(keyline){
        case 0:
          if(chin==0x43) Video.FrameCount+=60;
          if(chin==0x44)  Video.FrameCount-=60;
          break;
        case 1:
          if(chin==0x43) Video.ScreenCenter++;
          if(chin==0x44)  Video.ScreenCenter--;
          break;
        case 2:
          if(chin==0x43)cw++;
          if(chin==0x44) cw--;
          break;
        case 3:
          if(chin==0x43)ch++;
          if(chin==0x44) ch--;
          break;
        case 4:
          Screen.fill(32);
          if(chin==0x43)twidth++;
          if(chin==0x44) twidth--;
          break;
        case 5:
          if(chin==0x43) Video.ScreenTop++;
          if(chin==0x44)  Video.ScreenTop--;
          break;
        case 6:
          if(chin==0x43) Video.ScreenBottom++;
          if(chin==0x44)  Video.ScreenBottom--;
          break;
        case 7:
          if(chin==0x43)audio++;
          if(chin==0x44) audio--;
          if(key_sft){         
            setaudior(audio,40);
          }
          else  setaudiol(audio,40);
          break;
        case 8:
          if(chin==0x43)OCR2A++;
          if(chin==0x44) OCR2A--;
          break;
        case 9:
        case 10:
          if(chin==0x43)memory+=8;
          if(chin==0x44)memory-=8;
          break;
        } 
        break;
      }
      Video.ScreenWidth=twidth;  // adjust for changes made
      Video.ScreenMode=ch;
      keyline=keyline%10;
      setfixbigline((uint)Screenbuf,twidth,1+keyline*2,cw,ch); // recalculate selection line 
    }
  }
}


/*
// more generic coprocessor instructions and update routine for a selected line
 // setgoplist(setbigline( textbase, screenwidth, selline)); 
 volatile uchar bigline[]=// the graphics processor instructions as an array
 {
 // instruction format is  line#,flags,value(,value2...),
 // at line 95 set screen w=22 and character w=1 h=2, centring=17, in operation the set function changes the line count. 
 
 95,flag_gopcharwidth|flag_gopscreenwidth|flag_gopcharheight|flag_gopcenter,2,11, 3,(2*8)+1,
 // at line 119 set  screen w and character w,h centering also index since we changed screen width
 //(index=index into character buffer of first char next line.)
 119,flag_gopcharwidth|flag_gopindex |flag_gopscreenwidth|flag_gopcharheight|flag_gopcenter,0,5,5,15, 1,(16*8)+5,
 255,flag_gopreset ,gop_int(bigline)
 };
 
 volatile uchar * setbigline(uchar* textbase, uchar linelen, uchar selline){
 // while( Video.LineCount> Video.ScreenBottom);
 //  while( Video.LineCount< Video.ScreenBottom);
 
 bigline[0]= Video.ScreenTop+(selline*8)-1; // target line
 //bigline[1]=flag_gopcharwidth|flag_gopscreenwidth|flag_gopcharheight;
 //bigline[2]=1; //double wide char
 // bigline[3]=11;
 //  bigline[3]=(linelen+( Video.ScreenCenter/8)<22)?linelen:22-( Video.ScreenCenter/8);
 //bigline[4]=2; // double tall chars
 // bigline[5]=(7*8)+5;
 
 bigline[6]= Video.ScreenTop+(selline*8)+23; // end of target line
 // bigline[6]=flag_gopcharwidth|flag_gopindex |flag_gopscreenwidth|flag_gopcharheight;
 // bigline[7]=0; //single wide
 bigline[9]=((uint)textbase+((selline+1)*linelen))&0xff; // skipped half a line so fix pointer
 bigline[10]=((uint)textbase+((selline+1)*linelen))>>8;
 //bigline[11]=linelen;
 ///bigline[11]=1; //single tall chars
 // bigline[13]=(14*8)+5;
 //bigline[12]=5;//+256 =line261 good time to reset list
 //bigline[13]=flag_gopreset;
 //bigline[14]=(uchar)((int)bigline&0xff); // start of list
 //bigline[15]=(uchar)((int)bigline>>8);
 return((uchar *)bigline);
 } 
 
 
 */


